Index: src/Fl_Pixmap.cxx
===================================================================
--- src/Fl_Pixmap.cxx	(revision 10767)
+++ src/Fl_Pixmap.cxx	(working copy)
@@ -40,6 +40,10 @@
 #include <FL/Fl_Pixmap.H>
 #include <FL/Fl_Printer.H>
 
+#if !(defined(__APPLE__) || defined(WIN32))
+#include <X11/Xregion.h>
+#endif
+
 #include <stdio.h>
 #include "flstring.h"
 #include <ctype.h>
@@ -195,23 +199,32 @@
   int X, Y, W, H;
   if (pxm->prepare(XP, YP, WP, HP, cx, cy, X, Y, W, H)) return;
   if (pxm->mask_) {
-    // I can't figure out how to combine a mask with existing region,
-    // so cut the image down to a clipped rectangle:
-    int nx, ny; fl_clip_box(X,Y,W,H,nx,ny,W,H);
-    cx += nx-X; X = nx;
-    cy += ny-Y; Y = ny;
     // make X use the bitmap as a mask:
     XSetClipMask(fl_display, fl_gc, pxm->mask_);
-    int ox = X-cx; if (ox < 0) ox += pxm->w();
-    int oy = Y-cy; if (oy < 0) oy += pxm->h();
     XSetClipOrigin(fl_display, fl_gc, X-cx, Y-cy);
-  }
-  copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
-  if (pxm->mask_) {
+    // At this point, XYWH is the bounding box of the intersection between
+    // the current clip region and the (portion of the) pixmap we have to draw.
+    // The current clip region is often a rectangle. But, when the rounded top angle of a 
+    // window is moved above another window, expose events may create a complex clipping 
+    // region made of several (e.g., 10) rectangles. We have to draw only in the clipping
+    // region and also to mask out the image transparent pixels. We can't do that directly
+    // for a multi-rectangle clipping region because Xlib's clip is used by pxm->mask_.
+    // Thus, we draw each rectangle in the intersection of the clipping region and XHWH. 
+    Region r = XRectangleRegion(X,Y,W,H);
+    XIntersectRegion(r, clip_region(), r);
+    for (int i = 0; i < r->numRects; i++) {
+      int X2 = r->rects[i].x1; 
+      int Y2 = r->rects[i].y1;
+      W = r->rects[i].x2 - r->rects[i].x1; 
+      H = r->rects[i].y2 - r->rects[i].y1;
+      copy_offscreen(X2, Y2, W, H, pxm->id_, cx + (X2 - X), cy + (Y2 - Y));
+    }
+    XDestroyRegion(r);
     // put the old clip region back
     XSetClipOrigin(fl_display, fl_gc, 0, 0);
-    fl_restore_clip();
+    restore_clip();
   }
+  else copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
 }
 
 //------------------------------------------------------------------------------

